<?php

defined('_JEXEC') or die;

class DesignerShortcodes
{
    public static $shortcodes = array();
    public static $filters = array();

    public static function atts($defaults, $atts) {
        $atts = (array)$atts;
        $ret = array();
        foreach($defaults as $name => $default) {
            if (array_key_exists($name, $atts))
                $ret[$name] = $atts[$name];
            else
                $ret[$name] = $default;
        }
        return $ret;
    }

    public static function getRegexp($tag) {
        return '\\[(\\[?)' . "($tag)" . '(?![\\w-])([^\\]\\/]*(?:\\/(?!\\])[^\\]\\/]*)*?)(?:(\\/)\\]|\\](?:([^\\[]*+(?:\\[(?!\\/\\2\\])[^\\[]*+)*+)\\[\\/\\2\\])?)(\\]?)';
    }

    public static function process($content) {
        foreach(DesignerShortcodes::$filters as $filter => $func)
            $content = call_user_func($func, $content);
        $content = DesignerShortcodes::buildShortCodes($content);
        return $content;
    }

    public static function buildShortCodes($content) {
        foreach(DesignerShortcodes::$shortcodes as $tag => $func) {
            $pattern = self::getRegexp($tag);
            $content = preg_replace_callback( "/$pattern/s", 'DesignerShortcodes::replacer', $content );
        }
        return $content;
    }

    public static function replacer($matches) {

        $tag = $matches[2];
        $attr = self::parseAttr( $matches[3] );

        if (isset($matches[5])) {
            return $matches[1] . call_user_func( DesignerShortcodes::$shortcodes[$tag], $attr, $matches[5], $tag ) . $matches[6];
        } else {
            return $matches[1] . call_user_func( DesignerShortcodes::$shortcodes[$tag], $attr, null,  $tag ) . $matches[6];
        }
    }

    public static function parseAttr($text) {
        $atts = array();
        $pattern = '/(\w+)\s*=\s*"([^"]*)"(?:\s|$)|(\w+)\s*=\s*\'([^\']*)\'(?:\s|$)|(\w+)\s*=\s*([^\s\'"]+)(?:\s|$)|"([^"]*)"(?:\s|$)|(\S+)(?:\s|$)/';
        $text = preg_replace("/[\x{00a0}\x{200b}]+/u", " ", $text);
        if (preg_match_all($pattern, $text, $match, PREG_SET_ORDER)) {
            foreach ($match as $m) {
                if (!empty($m[1]))
                    $atts[strtolower($m[1])] = stripcslashes($m[2]);
                elseif (!empty($m[3]))
                    $atts[strtolower($m[3])] = stripcslashes($m[4]);
                elseif (!empty($m[5]))
                    $atts[strtolower($m[5])] = stripcslashes($m[6]);
                elseif (isset($m[7]) and strlen($m[7]))
                    $atts[] = stripcslashes($m[7]);
                elseif (isset($m[8]))
                    $atts[] = stripcslashes($m[8]);
            }
        } else {
            $atts = ltrim($text);
        }
        return $atts;
    }

}

function button_styling($atts, $content='') {
    extract(DesignerShortCodes::atts(array(
        'link' => '/',
        'type' => 'default',
        'style' => '',
        'size' => '',
        'icon' => ''
    ), $atts));

    $classNames = 'bd-button';
    $linkContent = $content;
    $styles = array('default' => 'btn-default', 'primary' => 'btn-primary', 'success' => 'btn-success',
        'info' => 'btn-info', 'warning' => 'btn-warning', 'danger' => 'btn-danger', 'link' => 'btn-link');
    $sizes = array('large' => 'btn-lg', 'small' => 'btn-sm', 'xsmall' => 'btn-xs');

    if ($type === 'bootstrap') {
        $classNames = 'btn';
        array_key_exists(strtolower($style), $styles) ? $classNames .= ' ' . $styles[strtolower($style)] : '';
        array_key_exists(strtolower($size), $sizes) ? $classNames .= ' ' . $sizes[strtolower($size)] : '';
    }

    if ($icon !== '') {
        $linkContent = '<span class="' . $icon . '">&nbsp;</span>' . $linkContent;
    }

    return '<a class="' . $classNames . '" href="' . $link . '">' . $linkContent . '</a>';
}

DesignerShortcodes::$shortcodes['button'] = 'button_styling';

function googlemap_styling($atts){
   extract(DesignerShortCodes::atts(array(
        'address' => '',
        'zoom' => '',
        'map_type' => '',
        'version' => '?output=embed',
        'language' => '',
        'tooltip' => '&iwloc=near',
        'style' => 'height:300px;width:100%',
    ), $atts));

    $languages = array(
        "basque" => "eu",
        "catalan" => "ca",
        "croatian" => "hr",
        "czech" => "cs",
        "danish" => "da",
        "dutch" => "nl",
        "english" => "en",
        "finnish" => "fi",
        "german" => "de",
        "galacian" => "gl",
        "greek" => "el",
        "hindi" => "hi",
        "indonesian" => "id",
        "italian" => "it",
        "japanese" => "ja",
        "norwegian" => "no",
        "nyorsk" => "nn",
        "portuguese" => "pt",
        "raeto-romance" => "rm",
        "russian" => "ru",
        "serbian" => "sr",
        "slovak" => "sk",
        "slovenian" => "sl",
        "spanish" => "es",
        "swedish" => "sv",
        "thai" => "th",
        "turkish" => "tr",
        "ukrainian" => "uk",
        "vietnamese" => "vi"
    );

    if ($version !== ''){
        switch ($version) {
                  case "1":
                    $version = '?output=svembed';
                    break;
                  default:
                    $version = '?output=embed';
                }
    }

    if ($address !== '') {
          $address = '&q=' . $address;
      }

    if ($zoom !== ''){
        $num = (int) $zoom;
        if ($num>0){
            $zoom = '&z=' . $num;
        }
        else{
            $zoom = '';
        }
    }

    if ($map_type !== ''){
        switch ($map_type) {
          case "road":
            $map_type = '&t=m';
            break;
          case "satelite":
            $map_type = '&t=k';
            break;
          case "hybrid":
          if ($version === '?output=svembed') {
                $map_type = '&t=h';
                break;
              }
          case "terrain":
              if ($version === '?output=svembed'){
                  $map_type = '&t=p';
                  break;
              }
          default:
            $map_type = '';
        }
    }

    if ($tooltip!==''){
        if ($version === '?output=svembed'){
            switch ($tooltip) {
              case "yes":
                $tooltip = '';
                break;
              default:
                $tooltip = '&iwloc=near';
            }
        }
        else {
            $tooltip = '';
        }
    }

    if ($language!==''){
        if ($version === '?output=svembed' && array_key_exists($language, $languages)){
          $language = '&hl=' . $languages[$language];
        }
        else {
          $language = '';
        }
    }

    $divs = '<div style="' . $style . '"><div class="embed-responsive" style="height: 100%; width: 100%;">';
    $iframe = '<iframe class="embed-responsive-item" src="http://maps.google.com/maps' . $version . $address . $zoom . $map_type . $language . $tooltip . '"></iframe>';
    $divEnd = '</div>';

    return $divs . $iframe . $divEnd . $divEnd;
}

DesignerShortcodes::$shortcodes['googlemap'] = 'googlemap_styling';

// [box css="" full_width="yes|no" content_width="yes|no"]content with shortcodes[/box]
function box_styling($atts, $content='') {
    extract(DesignerShortCodes::atts(array(
        'css' =>'',
        'full_width' => 'no',
        'content_width' => 'yes'
    ), $atts));
    $result = '';
    if ('yes' === $full_width) {
        $result .= '</div>';
    }
    $result .= '<div';
    if ($css !== '') {
        $result .= ' style="' . $css .'"';
    }
    $result .= '>';
    if ('yes' === $content_width) {
        $result .= '<div class="bd-container-inner">';
    }
    $result .= $content;
    if ('yes' === $content_width) {
        $result .= '</div>';
    }
    $result .= '</div>';
    if ('yes' === $full_width) {
        $result .= '<div class="bd-container-inner">';
    }
    return $result;
}
DesignerShortcodes::$shortcodes['box'] = 'box_styling';

// [[video link="https://www.youtube.com/watch?v=f20ym8X-9IU" autoplay="yes" loop="yes" title="no" lightBar="yes" style="width: 300px"][/video]
function video_styling($atts, $content='') {
    extract(DesignerShortCodes::atts(array(
        'link' => '/',
        'autoplay' => 'no',
        'loop' => 'no',
        'title' => 'yes',
        'light_control_bar' => 'no',
        'show_control_bar' => 'show',
        'css' => ''
    ), $atts));

    $isYouTube = strrpos($link, 'youtube');
    $isVimeo = strrpos($link, 'vimeo');

    if ($isYouTube !== false) {
        list(, $id) = explode('=', $link);
        list($id,) = explode('&', $id);
        $url = 'https://www.youtube.com/embed/' . $id . '?';

        if ($autoplay === 'yes')
            $url .= 'autoplay=1&';

        if ($title === 'no')
            $url .= 'showinfo=1&';

        if (light_control_bar === 'yes')
            $url .= 'theme=light&';

        if ($loop === 'yes')
            $url .= 'loop=1&playlist=' . $id . ' ';

        if ($show_control_bar === 'autohide')
            $url .= 'autohide=1&';
        else if ($show_control_bar === 'hide')
             $url .= 'controls=0&';

        $iframe = '<iframe src="' . $url . '"></iframe>';
    } else if ($isVimeo !== false) {
       $id = end(explode('/', $link));
       $url = 'https://player.vimeo.com/video/' . $id . '?';

       if ($autoplay === 'yes')
           $url .= 'autoplay=1&';

       if ($title === 'no')
           $url .= 'title=1&';

       if (light_control_bar === 'yes')
           $url .= 'color=ffffff&';

       if ($loop === 'yes')
           $url .= 'loop=1';

       $iframe = '<iframe src="' . $url . '"></iframe>';
    }

    return '<div class="embed-responsive embed-responsive-16by9" style="' . $css . '">' . $iframe . '</div>';
}
DesignerShortcodes::$shortcodes['video'] = 'video_styling';

function textGroup_styling($atts, $content='') {
    extract(DesignerShortCodes::atts(array(
        'image' => '',
        'image_link' => '',
        'image_position' => 'left',
        'image_width' => '',
        'image_height' => '',
        'image_css' => '',
        'header' => '',
        'header_tag' => 'h4',
        'header_css' => '',
        'css' => ''
    ), $atts));

    $image_positions = array('left' => 'pull-left', 'right' => 'pull-right', 'top' => 'top', 'bottom' => 'bottom', 'middle' => 'middle');

    $headers = array('h1' => 'h1', 'h2' => 'h2', 'h3' => 'h3', 'h4' => 'h4', 'h5' => 'h5', 'h6' => 'h6');
    array_key_exists(strtolower($header_tag), $headers) ? $_header = $headers[strtolower($header_tag)] : $_header = 'h4';
    $_header = $header !== '' ? '<' . $_header . ' class="media-heading"' .
                    ($header_css !== '' ? ' style="' . $header_css . '"' : '') . '>' . $header . '</' . $_header . '>' : '';

    $_ip = strtolower($image_position);

    $_i = '';
    if ($image !== '') {
        $_iWidth = $image_width !== '' ? ' width="' . ($image_width) . '"' : '';
        $_iHeight = $image_height !== '' ? ' height="' . ($image_height) . '"' : '';
        $_i = '<img class="  media-object img-responsive"' . ($image_css !== '' ? ' style="' . $image_css . '"' : '')
            . ' src="' . $image . '"' . $_iWidth . $_iHeight . '/>';
        if (array_key_exists($_ip, $image_positions)) {
            if ($_ip === 'left' || $_ip === 'right') {
                $_i = '<a class="' . $image_positions[$_ip] . '" href="' . ($image_link) . '">' . $_i . '</a>';
            } else {
                $_i = '<a href="' . ($image_link) . '">' . $_i . '</a>';
            }
        }
    }

    $_c = '<div class="media"' . ($css !== '' ? ' style="' . $css . '"' : '') . '>';

    if ($_ip === 'middle') {
        return $_c . '<div class="media-body">' . $_header . $_i . $content . '</div></div>';
    }

    $_content = '<div class="media-body">' . $_header . $content . '</div>';

    if ($_ip !== 'bottom') {
        return $_c . $_i . $_content . '</div>';
    }

    return $_c . $_content . $_i . '</div>';
}

DesignerShortcodes::$shortcodes['text_group'] = 'textGroup_styling';

class ThemeColumns {
    /*
        [column width_lg="6" width="8" width_sm="12" width_xs="6"] Your Content Here [/column]
        [one_half last] 1/2 [/one_half]
        [one_third] 1/3 [/one_third]
        [two_third last] 2/3 [/two_third]
        [one_fourth]  1/4 [/one_fourth]
        [three_fourth] 3/4 [/three_fourth]
    */
    public static function column($atts, $content = '') {
        extract(DesignerShortcodes::atts(array(
            'width_lg' => '',
            'width' => '4',
            'width_sm' => '',
            'width_xs' => '',
            'css' => '',
            'last' => false
        ), $atts));

        if (false === $last && is_array($atts) && !empty($atts)) {
            foreach( $atts as $key => $value) {
                if (is_numeric($key) && 'last' ===  $value) {
                    $last = true;
                    break;
                }
            }
        }

        $classes = array();
        if (intval($width_lg) > 0) {
            $classes[] = 'col-lg-' . $width_lg;
        }
        if (intval($width) > 0) {
            $classes[] = 'col-md-' . $width;
        }
        if (intval($width_sm) > 0) {
            $classes[] = 'col-sm-' . $width_sm;
        }
        if (intval($width_xs) > 0) {
            $classes[] = 'col-xs-' . $width_xs;
        }

        $result = '<!--Column="' . implode(' ', $classes) . '"-->';
        if ($css !== '') {
            $result .= '<div style="' . $css .'">' . $content . '</div>';
        } else {
            $result .=  $content;
        }
        $result .=  '<!--/Column' . (false !== $last ? 'Last' : '') .  '-->';

        return $result;
    }

    public static function one_half($atts, $content = '') {
        $atts['width'] = "12";
        return ThemeColumns::column($atts, $content);
    }

    public static function one_third($atts, $content = '') {
        $atts['width'] = "8";
        return ThemeColumns::column($atts, $content);
    }

    public static function two_third($atts, $content = '') {
        $atts['width'] = "16";
        return ThemeColumns::column($atts, $content);
    }

    public static function one_fourth($atts, $content = '') {
        $atts['width'] = "6";
        return ThemeColumns::column($atts, $content);
    }

    public static function three_fourth($atts, $content = '') {
        $atts['width'] = "18";
        return ThemeColumns::column($atts, $content);
    }

    public static $row = false;
    public static function filter($content) {

        $orig_shortcode_tags = DesignerShortcodes::$shortcodes;

        DesignerShortcodes::$shortcodes = array();
        DesignerShortcodes::$shortcodes['column'] = 'ThemeColumns::column';
        DesignerShortcodes::$shortcodes['one_half'] = 'ThemeColumns::one_half';
        DesignerShortcodes::$shortcodes['one_third'] = 'ThemeColumns::one_third';
        DesignerShortcodes::$shortcodes['two_third'] = 'ThemeColumns::two_third';
        DesignerShortcodes::$shortcodes['one_fourth'] = 'ThemeColumns::one_fourth';
        DesignerShortcodes::$shortcodes['three_fourth'] = 'ThemeColumns::three_fourth';
        $content = DesignerShortcodes::buildShortCodes($content);

        DesignerShortcodes::$shortcodes = $orig_shortcode_tags;

        ThemeColumns::$row = false;
        $content = preg_replace('/(<!--\/Column)(?:Last){0,1}(-->)(?![\s\S]*<!--\/Column)/s', '$1Last$2', $content, 1);
        return  preg_replace_callback('|<!--Column(="[^"]*")-->([\s\S]*?)<!--\/Column(Last){0,1}-->|s','ThemeColumns::callback', $content);
    }

    public static function callback($matches)
    {
        $result = '';
        if (!ThemeColumns::$row) {
            $result .= '<div class="row">';
            ThemeColumns::$row = true;
        }
        $result .= '<div class'.$matches[1] . '>'.$matches[2] .'</div>';
        if (isset($matches[3])) {
            $result .= '</div>';
            ThemeColumns::$row = false;
        }
        return $result;
    }
}

DesignerShortcodes::$filters[] = 'ThemeColumns::filter';